@@ -1,3 +1,4 @@
+/* $XFree86: xc/extras/freetype2/src/cff/cffgload.c,v 1.5 2007/04/05 16:08:58 tsi Exp $ */
 /***************************************************************************/
 /*                                                                         */
 /*  cffgload.c                                                             */
@@ -15,9 +16,6 @@
 /*                                                                         */
 /***************************************************************************/
 
-/* $XFree86: xc/extras/freetype2/src/cff/cffgload.c,v 1.4 2004/04/26 16:15:54 dawes Exp $ */
-
-
 #include <ft2build.h>
 #include FT_INTERNAL_DEBUG_H
 #include FT_INTERNAL_CALC_H
@@ -2288,7 +2286,7 @@
   FT_LOCAL_DEF( FT_Error )
   cff_slot_load( CFF_GlyphSlot  glyph,
                  CFF_Size       size,
-                 FT_Int         glyph_index,
+                 FT_UInt        glyph_index,
                  FT_Int32       load_flags )
   {
     FT_Error      error;
@@ -2334,7 +2332,7 @@
 
         error = sfnt->load_sbit_image( face,
                                        (FT_ULong)size->strike_index,
-                                       (FT_UInt)glyph_index,
+                                       glyph_index,
                                        (FT_Int)load_flags,
                                        stream,
                                        &glyph->root.bitmap,
@@ -2397,7 +2395,12 @@
       /* subsetted font, glyph_indices and CIDs are identical, though */
       if ( cff->top_font.font_dict.cid_registry != 0xFFFFU &&
            cff->charset.cids )
-        glyph_index = cff->charset.cids[glyph_index];
+      {
+        if ( glyph_index < cff->charset.max_cid )
+          glyph_index = cff->charset.cids[glyph_index];
+        else
+          glyph_index = 0;
+      }
 
       cff_decoder_init( &decoder, face, size, glyph, hinting,
                         FT_LOAD_TARGET_MODE( load_flags ) );
