@@ -31,7 +31,7 @@
    promote the sale, use or other dealings in this Software without
    prior written authorization.
 */
-/* $XFree86: xc/programs/Xserver/hw/darwin/quartz/quartzKeyboard.c,v 1.5 2006/01/09 14:59:48 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/darwin/quartz/quartzKeyboard.c,v 1.6 2007/04/03 00:21:09 tsi Exp $ */
 
 #include "quartzCommon.h"
 
@@ -146,7 +146,7 @@ const static struct {
 };
 
 unsigned int
-DarwinSystemKeymapSeed (void)
+DarwinModeSystemKeymapSeed (void)
 {
     static unsigned int seed;
 
@@ -371,6 +371,12 @@ DarwinModeReadSystemKeymap (darwinKeyboardInfo *info)
 
 #else /* !HAS_KL_API */
 
+unsigned int
+DarwinModeSystemKeymapSeed (void)
+{
+    return 0;
+}
+
 Bool
 DarwinModeReadSystemKeymap (darwinKeyboardInfo *info)
 {
