@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/loader/xf86sym.c,v 1.260 2006/03/21 03:56:26 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/loader/xf86sym.c,v 1.267 2007/05/22 16:49:47 tsi Exp $ */
 
 /*
  *
@@ -23,7 +23,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 /*
- * Copyright (c) 1997-2005 by The XFree86 Project, Inc.
+ * Copyright (c) 1997-2006 by The XFree86 Project, Inc.
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -105,7 +105,7 @@
 #include "xf86_ansic.h"
 #include "xisb.h"
 #include "vbe.h"
-#ifndef __OpenBSD__
+#if defined(__sparc__)
 #include "xf86sbusBus.h"
 #endif
 #include "compiler.h"
@@ -149,7 +149,11 @@ extern void *__remq(long, long);
 extern void *__remqu(long, long);
 #endif
 
-#if defined(__sparc__) && defined(__FreeBSD__)
+#undef NO_HARD_QUAD
+#if defined(CSRG_BASED) && defined(__sparc__) && defined(__GNUC__) && \
+    ((__GNUC__ > 3) || ((__GNUC__ == 3) && (__GNUC_MINOR__ >= 2)) && \
+    (defined(__arch64__) || defined(__sparcv9) || defined(__sparcv9__)))
+#define NO_HARD_QUAD
 extern float _Qp_qtos(unsigned int *);
 extern double _Qp_qtod(unsigned int *);
 extern unsigned long long _Qp_qtoux(unsigned int *);
@@ -231,10 +235,6 @@ extern unsigned long long __xtoull(long);
 #pragma weak __xtoull
 #endif
 
-#if defined(__arm__) && defined(__linux__)
-#include <sys/io.h>
-#endif
-
 #if defined(__powerpc__) && (defined(Lynx) || defined(linux))
 void _restf14();
 void _restf17();
@@ -263,8 +263,9 @@ void _savef27();
 void _savef28();
 void _savef29();
 
-/* even if we compile without -DNO_INLINE we still provide
- * the usual port i/o functions for module use
+/*
+ * Even if we compile without -DNO_INLINE we still provide
+ * the usual port i/o functions for module use.
  */
 
 extern volatile unsigned char *ioBase;
@@ -284,6 +285,14 @@ extern unsigned long ldl_brx(volatile unsigned char *, int);
 extern unsigned short ldw_brx(volatile unsigned char *, int);
 #endif
 
+/*
+ * For propolice/gcc stack protector.
+ */
+extern long __guard[8];
+extern void __stack_smash_handler(char func[], int damaged);
+#pragma weak __guard
+#pragma weak __stack_smash_handler
+
 /* XFree86 things */
 
 LOOKUP xfree86LookupTab[] = {
@@ -663,7 +672,7 @@ LOOKUP xfree86LookupTab[] = {
     SYMFUNC(xf86AddModuleInfo)
     SYMFUNC(xf86DeleteModuleInfo)
 
-#if defined(__sparc__) && !defined(__OpenBSD__)
+#if defined(__sparc__)
     /* xf86sbusBus.c */
     SYMFUNC(xf86ParseSbusBusString)
     SYMFUNC(xf86CompareSbusBusString)
@@ -677,7 +686,12 @@ LOOKUP xfree86LookupTab[] = {
     SYMFUNC(xf86UnmapSbusMem)
     SYMFUNC(xf86SbusHideOsHwCursor)
     SYMFUNC(xf86SbusSetOsHwCursorCmap)
+    SYMFUNC(xf86SbusSetOsHwCursorImage)
+    SYMFUNC(xf86SbusSetOsHwCursor)
+    SYMFUNC(xf86SbusSetOsHwCursorPosition)
+    SYMFUNC(xf86SbusSetOsHwCursorHotSpot)
     SYMFUNC(xf86SbusHandleColormaps)
+    SYMFUNC(xf86SbusSaveScreen)
     SYMFUNC(sparcPromInit)
     SYMFUNC(sparcPromClose)
     SYMFUNC(sparcPromGetProperty)
@@ -1015,6 +1029,7 @@ LOOKUP xfree86LookupTab[] = {
     SYMFUNC(xf86pow)
     SYMFUNC(xf86printf)
     SYMFUNC(xf86qsort)
+    SYMFUNC(xf86rand)
     SYMFUNC(xf86read)
     SYMFUNC(xf86realloc)
     SYMFUNC(xf86remove)
@@ -1283,15 +1298,19 @@ LOOKUP xfree86LookupTab[] = {
     SYMFUNC(inl)
 #endif
 
-#ifdef __FreeBSD__
-#if defined(__sparc__)
+#if defined(NO_HARD_QUAD)
     SYMFUNC(_Qp_qtos)
     SYMFUNC(_Qp_qtod)
     SYMFUNC(_Qp_qtoux)
     SYMFUNC(_Qp_uitoq)
     SYMFUNC(_Qp_dtoq)
 #endif
-#endif
+
+    /*
+     * For propolice/gcc stack protector.
+     */
+    SYMFUNC(__stack_smash_handler)
+    SYMVAR(__guard)
 
     /* Some variables. */
 
