@@ -1,4 +1,3 @@
-/* $Xorg: Screen.h,v 1.3 2000/08/17 19:53:28 cpqbld Exp $ */
 /*
 
 Copyright 1993 by Davor Matic
@@ -12,7 +11,7 @@ the suitability of this software for any purpose.  It is provided "as
 is" without express or implied warranty.
 
 */
-/* $XFree86: xc/programs/Xserver/hw/xnest/Screen.h,v 1.2 2003/11/16 05:05:20 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xnest/Screen.h,v 1.5 2007/01/23 18:03:12 tsi Exp $ */
 
 #ifndef XNESTSCREEN_H
 #define XNESTSCREEN_H
@@ -21,7 +20,7 @@ extern Window xnestDefaultWindows[MAXSCREENS];
 extern Window xnestScreenSaverWindows[MAXSCREENS];
 
 ScreenPtr xnestScreen(Window window);
-Bool xnestOpenScreen(int index, ScreenPtr pScreen, int argc, char *argv[]);
+Bool xnestOpenScreen(int index, ScreenPtr pScreen, const int argc, const char *argv[]);
 Bool xnestCloseScreen(int index, ScreenPtr pScreen);
 
 #endif /* XNESTSCREEN_H */
